@@ -2,7 +2,7 @@ package Time::Zone::Olson;
 
 use 5.010;
 use strict;
-use warnings FATAL => 'all';
+use warnings;
 
 use FileHandle();
 use File::Spec();
@@ -10,8 +10,9 @@ use Config;
 use Carp();
 use English qw( -no_match_vars );
 use DirHandle();
+use POSIX();
 
-our $VERSION = 0.07;
+our $VERSION = 0.08;
 
 sub _SIZE_OF_TZ_HEADER                     { return 44 }
 sub _SIZE_OF_TRANSITION_TIME_V1            { return 4 }
@@ -666,7 +667,7 @@ sub _positive_time_local {
     }
     $time += $localtime[ _LOCALTIME_SECOND_INDEX() ];
     my ( $isdst, $gmtoff, $abbr ) =
-      $self->_get_isdst_gmtoff_abbr_converted_from_gmt_base($time);
+      $self->_get_isdst_gmtoff_abbr_calculating_for_time_local($time);
     $time -= $gmtoff;
 
     return $time;
@@ -770,40 +771,14 @@ sub _negative_time_local {
     }
     $time += $localtime[ _LOCALTIME_SECOND_INDEX() ];
     my ( $isdst, $gmtoff, $abbr ) =
-      $self->_get_isdst_gmtoff_abbr_converted_from_gmt_base($time);
+      $self->_get_isdst_gmtoff_abbr_calculating_for_time_local($time);
     $time -= $gmtoff;
 
     return $time;
 }
 
-sub _get_isdst_gmtoff_abbr_converted_from_gmt_base {
-    my ( $self, $time ) = @_;
-    if ( defined $self->offset() ) {
-        return ( 0, $self->offset() * _SECONDS_IN_ONE_MINUTE(), q[] );
-    }
-    my ( $isdst, $gmtoff, $abbr );
-    my $tz = $self->timezone();
-    $self->_read_tzfile();
-    my $transition_index = 0;
-    my $transition_time_found;
-    my $first_transition_time;
-  TRANSITION_TIME: foreach my $transition_time ( $self->transition_times() ) {
-
-        if ( !defined $first_transition_time ) {
-            $first_transition_time = $transition_time;
-        }
-        $transition_index += 1;
-        my $local_time_index = $self->{_tzdata}->{$tz}->{local_time_indexes}
-          ->[ $transition_index - 1 ];
-        my $local_time_type =
-          $self->{_tzdata}->{$tz}->{local_time_types}->[$local_time_index];
-        $transition_time += $local_time_type->{gmtoff};
-        if ( $transition_time > $time ) {
-            $transition_index -= 1;
-            $transition_time_found = 1;
-            last TRANSITION_TIME;
-        }
-    }
+sub _get_first_standard_time_type {
+    my ( $self, $tz ) = @_;
     my $first_standard_time_type;
     if ( defined $self->{_tzdata}->{$tz}->{local_time_types}->[0] ) {
         $first_standard_time_type =
@@ -820,6 +795,59 @@ sub _get_isdst_gmtoff_abbr_converted_from_gmt_base {
             last FIRST_STANDARD_TIME_TYPE;
         }
     }
+    return $first_standard_time_type;
+}
+
+sub _get_isdst_gmtoff_abbr_calculating_for_time_local {
+    my ( $self, $time ) = @_;
+    if ( defined $self->offset() ) {
+        return ( 0, $self->offset() * _SECONDS_IN_ONE_MINUTE(), q[] );
+    }
+    my ( $isdst, $gmtoff, $abbr );
+    my $tz = $self->timezone();
+    $self->_read_tzfile();
+    my $first_standard_time_type = $self->_get_first_standard_time_type($tz);
+    my $transition_index         = 0;
+    my $transition_time_found;
+    my $previous_offset = $first_standard_time_type->{gmtoff};
+    my $first_transition_time;
+  TRANSITION_TIME:
+    foreach my $transition_time_in_gmt ( $self->transition_times() ) {
+
+        if ( !defined $first_transition_time ) {
+            $first_transition_time = $transition_time_in_gmt;
+        }
+        my $local_time_index =
+          $self->{_tzdata}->{$tz}->{local_time_indexes}->[$transition_index];
+        my $local_time_type =
+          $self->{_tzdata}->{$tz}->{local_time_types}->[$local_time_index];
+        if ( $local_time_type->{gmtoff} < $previous_offset ) {
+            if (
+                ( $transition_time_in_gmt > $time - $previous_offset )
+                && ( $transition_time_in_gmt <=
+                    $time - $local_time_type->{gmtoff} )
+              )
+            {
+                $transition_time_found = 1;
+                last TRANSITION_TIME;
+            }
+            elsif (
+                $transition_time_in_gmt > $time - $local_time_type->{gmtoff} )
+            {
+                $transition_time_found = 1;
+                last TRANSITION_TIME;
+            }
+        }
+        else {
+            if ( $transition_time_in_gmt > $time - $local_time_type->{gmtoff} )
+            {
+                $transition_time_found = 1;
+                last TRANSITION_TIME;
+            }
+        }
+        $transition_index += 1;
+        $previous_offset = $local_time_type->{gmtoff};
+    }
     my $offset_found;
     if (
            ( defined $first_transition_time )
@@ -862,7 +890,7 @@ sub _get_isdst_gmtoff_abbr_converted_from_gmt_base {
     return ( $isdst, $gmtoff, $abbr );
 }
 
-sub _get_isdst_gmtoff_abbr_as_localtime {
+sub _get_isdst_gmtoff_abbr_calculating_for_local_time {
     my ( $self, $time ) = @_;
     my ( $isdst, $gmtoff, $abbr );
     if ( defined $self->offset() ) {
@@ -873,34 +901,19 @@ sub _get_isdst_gmtoff_abbr_as_localtime {
     my $transition_index = 0;
     my $transition_time_found;
     my $first_transition_time;
-  TRANSITION_TIME: foreach my $transition_time ( $self->transition_times() ) {
+  TRANSITION_TIME:
+    foreach my $transition_time_in_gmt ( $self->transition_times() ) {
 
         if ( !defined $first_transition_time ) {
-            $first_transition_time = $transition_time;
+            $first_transition_time = $transition_time_in_gmt;
         }
-        $transition_index += 1;
-        if ( $transition_time > $time ) {
-            $transition_index -= 1;
+        if ( $transition_time_in_gmt > $time ) {
             $transition_time_found = 1;
             last TRANSITION_TIME;
         }
+        $transition_index += 1;
     }
-    my $first_standard_time_type;
-    if ( defined $self->{_tzdata}->{$tz}->{local_time_types}->[0] ) {
-        $first_standard_time_type =
-          $self->{_tzdata}->{$tz}->{local_time_types}->[0];
-    }
-  FIRST_STANDARD_TIME_TYPE:
-    foreach
-      my $local_time_type ( @{ $self->{_tzdata}->{$tz}->{local_time_types} } )
-    {
-        if ( $local_time_type->{isdst} ) {
-        }
-        else {
-            $first_standard_time_type = $local_time_type;
-            last FIRST_STANDARD_TIME_TYPE;
-        }
-    }
+    my $first_standard_time_type = $self->_get_first_standard_time_type($tz);
     my $offset_found;
     if (   ( defined $first_transition_time )
         && ( $time < $first_transition_time ) )
@@ -945,7 +958,7 @@ sub local_offset {
         $time = time;
     }
     my ( $isdst, $gmtoff, $abbr ) =
-      $self->_get_isdst_gmtoff_abbr_as_localtime($time);
+      $self->_get_isdst_gmtoff_abbr_calculating_for_local_time($time);
     return int( $gmtoff / _SECONDS_IN_ONE_MINUTE() );
 }
 
@@ -956,7 +969,7 @@ sub local_time {
     }
 
     my ( $isdst, $gmtoff, $abbr ) =
-      $self->_get_isdst_gmtoff_abbr_as_localtime($time);
+      $self->_get_isdst_gmtoff_abbr_calculating_for_local_time($time);
     $time += $gmtoff;
 
     if (wantarray) {
@@ -1491,7 +1504,7 @@ Time::Zone::Olson - Provides an interface to the Olson timezone database
 
 =head1 VERSION
 
-Version 0.07
+Version 0.08
 
 =cut
 
@@ -1565,7 +1578,9 @@ This object method has the same signature as the 64 bit version of the C<localti
 
 =head2 time_local
 
-This object method has the same signature as the 64 bit version of the C<Time::Local::timelocal> function.  That is, it accepts C<(seconds, minutes, hours, day, month, year, wday, yday, isdst)> as parameters in a list and returns the correct UNIX time in seconds according to the current timezone for the object.  The timezone used to calculate the local time may be specified as a parameter to the L</new> method or via the L</timezone> method.  It may not return the same values as Time::Local::timelocal though, especially during timezone changes.
+This object method has the same signature as the 64 bit version of the C<Time::Local::timelocal> function.  That is, it accepts C<(seconds, minutes, hours, day, month, year, wday, yday, isdst)> as parameters in a list and returns the correct UNIX time in seconds according to the current timezone for the object.  The timezone used to calculate the local time may be specified as a parameter to the L</new> method or via the L</timezone> method. 
+
+During a time zone change such as +11 GMT to +10 GMT, there will be two possible UNIX times that can result in the same local time.  In this case, like C<Time::Local::timelocal>, this function will return the lower of the two times.
 
 =head2 transition_times
 
